import { createNestedMiddleware, } from "prisma-nested-middleware";
import { createAggregateMiddleware, createCountMiddleware, createDeleteManyMiddleware, createDeleteMiddleware, createFindFirstMiddleware, createFindManyMiddleware, createFindUniqueMiddleware, createIncludeMiddleware, createSelectMiddleware, createUpdateManyMiddleware, createUpdateMiddleware, createUpsertMiddleware, createWhereMiddleware, } from "./actionMiddleware";
export function createSoftDeleteMiddleware({ models, defaultConfig = {
    field: "deleted",
    createValue: Boolean,
    allowToOneUpdates: false,
}, }) {
    if (!defaultConfig.field) {
        throw new Error("prisma-soft-delete-middleware: defaultConfig.field is required");
    }
    if (!defaultConfig.createValue) {
        throw new Error("prisma-soft-delete-middleware: defaultConfig.createValue is required");
    }
    const modelConfig = {};
    Object.keys(models).forEach((model) => {
        const modelName = model;
        const config = models[modelName];
        if (config) {
            modelConfig[modelName] =
                typeof config === "boolean" && config ? defaultConfig : config;
        }
    });
    const middlewareByModel = Object.keys(modelConfig).reduce((acc, model) => {
        const config = modelConfig[model];
        return {
            ...acc,
            [model]: {
                delete: createDeleteMiddleware(config),
                deleteMany: createDeleteManyMiddleware(config),
                update: createUpdateMiddleware(config),
                updateMany: createUpdateManyMiddleware(config),
                upsert: createUpsertMiddleware(config),
                findFirst: createFindFirstMiddleware(config),
                findUnique: createFindUniqueMiddleware(config),
                findMany: createFindManyMiddleware(config),
                count: createCountMiddleware(config),
                aggregate: createAggregateMiddleware(config),
                where: createWhereMiddleware(config),
                include: createIncludeMiddleware(config),
                select: createSelectMiddleware(config),
            },
        };
    }, {});
    // before handling root params generate deleted value so it is consistent
    // for the query. Add it to root params and get it from scope?
    return createNestedMiddleware((params, next) => {
        var _a;
        const middleware = (_a = middlewareByModel[params.model || ""]) === null || _a === void 0 ? void 0 : _a[params.action];
        // apply middleware if it is found for model and action
        if (middleware)
            return middleware(params, next);
        return next(params);
    });
}
