"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSoftDeleteMiddleware = void 0;
const prisma_nested_middleware_1 = require("prisma-nested-middleware");
const actionMiddleware_1 = require("./actionMiddleware");
function createSoftDeleteMiddleware({ models, defaultConfig = {
    field: "deleted",
    createValue: Boolean,
    allowToOneUpdates: false,
}, }) {
    if (!defaultConfig.field) {
        throw new Error("prisma-soft-delete-middleware: defaultConfig.field is required");
    }
    if (!defaultConfig.createValue) {
        throw new Error("prisma-soft-delete-middleware: defaultConfig.createValue is required");
    }
    const modelConfig = {};
    Object.keys(models).forEach((model) => {
        const modelName = model;
        const config = models[modelName];
        if (config) {
            modelConfig[modelName] =
                typeof config === "boolean" && config ? defaultConfig : config;
        }
    });
    const middlewareByModel = Object.keys(modelConfig).reduce((acc, model) => {
        const config = modelConfig[model];
        return {
            ...acc,
            [model]: {
                delete: (0, actionMiddleware_1.createDeleteMiddleware)(config),
                deleteMany: (0, actionMiddleware_1.createDeleteManyMiddleware)(config),
                update: (0, actionMiddleware_1.createUpdateMiddleware)(config),
                updateMany: (0, actionMiddleware_1.createUpdateManyMiddleware)(config),
                upsert: (0, actionMiddleware_1.createUpsertMiddleware)(config),
                findFirst: (0, actionMiddleware_1.createFindFirstMiddleware)(config),
                findUnique: (0, actionMiddleware_1.createFindUniqueMiddleware)(config),
                findMany: (0, actionMiddleware_1.createFindManyMiddleware)(config),
                count: (0, actionMiddleware_1.createCountMiddleware)(config),
                aggregate: (0, actionMiddleware_1.createAggregateMiddleware)(config),
                where: (0, actionMiddleware_1.createWhereMiddleware)(config),
                include: (0, actionMiddleware_1.createIncludeMiddleware)(config),
                select: (0, actionMiddleware_1.createSelectMiddleware)(config),
            },
        };
    }, {});
    // before handling root params generate deleted value so it is consistent
    // for the query. Add it to root params and get it from scope?
    return (0, prisma_nested_middleware_1.createNestedMiddleware)((params, next) => {
        var _a;
        const middleware = (_a = middlewareByModel[params.model || ""]) === null || _a === void 0 ? void 0 : _a[params.action];
        // apply middleware if it is found for model and action
        if (middleware)
            return middleware(params, next);
        return next(params);
    });
}
exports.createSoftDeleteMiddleware = createSoftDeleteMiddleware;
